@@ -307,6 +307,7 @@ void MainWindow::showHighscores()
     KScoreDialog scoreDialog(KScoreDialog::Name | KScoreDialog::Time, this);
     scoreDialog.addField(KScoreDialog::Custom1, i18n("Moves Penalty"), "moves");
     scoreDialog.addLocalizedConfigGroupNames(KGameDifficulty::localizedLevelStrings()); //Add all the translations of the group names
+    scoreDialog.setConfigGroupWeights(KGameDifficulty::levelWeights());
     scoreDialog.setConfigGroup(KGameDifficulty::localizedLevelString());
     scoreDialog.exec();
 }
@@ -479,11 +480,12 @@ void MainWindow::checkIfGameEnded()
     KScoreDialog scoreDialog(KScoreDialog::Name | KScoreDialog::Time, this);
     scoreDialog.addField(KScoreDialog::Custom1, i18n("Moves Penalty"), "moves");
     scoreDialog.addLocalizedConfigGroupNames(KGameDifficulty::localizedLevelStrings()); //Add all the translations of the group names
+    scoreDialog.setConfigGroupWeights(KGameDifficulty::levelWeights());
     scoreDialog.setConfigGroup(KGameDifficulty::localizedLevelString());
     bool madeIt = scoreDialog.addScore(scoreInfo);
     if (!madeIt) {
-        QString comment = i18n("Your score was %1, you did not make it "
-                               "to the highScore list", score);
+        QString comment = i18np("Your score was %1, you did not make it to the highScore list",
+				"Your score was %1, you did not make it to the highScore list", score);
         scoreDialog.setComment(comment);
     }
     scoreDialog.exec();
