@@ -17,15 +17,13 @@
 #include "cardmaps.h"
 #include "speeds.h"
 #include "version.h"
+#include "view.h"
 #include "patsolve/patsolve.h"
 
-#include <kconfiggroup.h>
-#include <kdebug.h>
-#include <klocalizedstring.h>
-#include <krandom.h>
-#include <kstandarddirs.h>
-#include <ksharedconfig.h>
+#include <cassert>
+#include <cmath>
 
+#include <QStyleOptionGraphicsItem>
 #include <QtCore/QMutex>
 #include <QtCore/QString>
 #include <QtCore/QThread>
@@ -36,15 +34,40 @@
 #include <QtSvg/QSvgRenderer>
 #include <QtXml/QDomDocument>
 
-#include <assert.h>
-#include <math.h>
+#include <KTemporaryFile>
+#include <kconfiggroup.h>
+#include <kdebug.h>
+#include <klocalizedstring.h>
+#include <krandom.h>
+#include <kstandarddirs.h>
+#include <ksharedconfig.h>
+#include <kio/netaccess.h>
 
 #define DEBUG_HINTS 0
 
+#if 0
+inline void myassert_fail (__const char *__assertion, __const char *__file,
+                           unsigned int __line, __const char *__function)
+{
+   QString tmp = PatienceView::instance()->dscene()->save_it();
+   fprintf(stderr, "SAVE %s.saved\n", qPrintable(tmp));
+   KIO::NetAccess::upload(tmp, tmp + ".saved", PatienceView::instance());
+   KIO::NetAccess::del(tmp, PatienceView::instance());
+   __assert_fail(__assertion, __file, __line, __function);
+}
+
+# define myassert(expr)                                                \
+  ((expr)                                                               \
+   ? __ASSERT_VOID_CAST (0)                                             \
+   : myassert_fail (__STRING(expr), __FILE__, __LINE__, __ASSERT_FUNCTION) )
+#endif
+
+#define myassert assert
+
 // ================================================================
 //                         class MoveHint
 
-MoveHint::MoveHint(Card *card, Pile *to, qint8 prio)
+MoveHint::MoveHint(Card *card, Pile *to, int prio)
 {
     m_card         = card;
     m_to           = to;
@@ -137,7 +160,7 @@ public:
     bool _won;
     quint32 _id;
     bool _gameRecorded;
-    QGraphicsItem *wonItem;
+    QGraphicsPixmapItem *wonItem;
     bool gothelp;
     bool toldAboutLostGame;
     // we need a flag to avoid telling someone the game is lost
@@ -153,6 +176,8 @@ public:
     bool hasScreenRect;
 
     QTimer *demotimer;
+    QTimer *stateTimer;
+    QTimer *autoDropTimer;
     bool demo_active;
     bool stop_demo_next;
     qreal m_autoDropFactor;
@@ -160,48 +185,68 @@ public:
     static QSvgRenderer *backren;
     QPixmap backPix;
     bool initialDeal;
+    qreal offx;
+    qreal offy;
+    bool m_autoDropOnce;
+    QList<State*> redoList;
+    QList<State*> undoList;
 };
 
+int DealerScene::getMoves() const
+{
+    return d->undoList.count();
+}
+
 void DealerScene::takeState()
 {
-    // kDebug(11111) << "takeState" << waiting();
+    //kDebug(11111) << "takeState" << waiting();
+
+    if ( !demoActive() )
+    {
+        //kDebug(11111) << "clear in takeState";
+        d->winMoves.clear();
+        clearHints();
+    }
+    if ( waiting() )
+    {
+        d->stateTimer->start();
+        return;
+    }
 
     QList<QGraphicsItem *> list = items();
-    for (QList<QGraphicsItem *>::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for (QList<QGraphicsItem *>::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
     {
         if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId ) {
             Card *c = static_cast<Card*>(*it);
-            if (c->animated()) {
-                QTimer::singleShot(100, this, SLOT(takeState()));
+            if (c->animated())
+            {
+                d->stateTimer->start();
                 return;
             }
         }
     }
 
+    d->stateTimer->stop();
+
     State *n = getState();
 
-    if (!undoList.count()) {
+    if (!d->undoList.count()) {
         emit updateMoves();
-        undoList.append(n);
+        d->undoList.append(n);
     } else {
-        State *old = undoList.last();
+        State *old = d->undoList.last();
 
         if (*old == *n) {
             delete n;
             n = 0;
         } else {
             emit updateMoves();
-            undoList.append(n);
+            d->undoList.append(n);
         }
     }
 
     if (n) {
         d->initialDeal = false;
-        if ( !demoActive() )
-        {
-            kDebug(11111) << "clear in takeState";
-            d->winMoves.clear();
-        }
         if (isGameWon()) {
             won();
             d->toldAboutWonGame = true;
@@ -218,13 +263,14 @@ void DealerScene::takeState()
         }
 
         if ( d->m_solver && !demoActive() && !waiting() )
-            d->updateSolver->start();
+            startSolver();
 
+        if (!demoActive() && !waiting())
+            d->autoDropTimer->start( speedUpTime( TIME_BETWEEN_MOVES ) );
+
+        emit undoPossible(d->undoList.count() > 1);
+        emit redoPossible(d->redoList.count() > 1);
     }
-    if (!demoActive() && !waiting())
-        QTimer::singleShot( speedUpTime( TIME_BETWEEN_MOVES ), this, SLOT(startAutoDrop()));
-
-    emit undoPossible(undoList.count() > 1);
 }
 
 bool DealerScene::isInitialDeal() const { return d->initialDeal; }
@@ -286,8 +332,8 @@ void DealerScene::saveGame(QDomDocument &doc)
         if (!n->gameData.isEmpty())
             state.setAttribute("data", n->gameData);
         QDomElement cards = doc.createElement("cards");
-        for (QValueList<CardState>::ConstIterator it2 = n->cards.begin();
-             it2 != n->cards.end(); ++it2)
+        for (QValueList<CardState>::ConstIterator it2 = n->cards.constBegin();
+             it2 != n->cards.constEnd(); ++it2)
         {
             QDomElement item = doc.createElement("item");
             (*it2).fillNode(item);
@@ -313,7 +359,7 @@ void DealerScene::openGame(QDomDocument &doc)
     QList<QGraphicsItem *> list = items();
 
     CardList cards;
-    for (QList<QGraphicsItem *>::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for (QList<QGraphicsItem *>::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
         if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId )
             cards.append(static_cast<Card*>(*it));
 
@@ -361,10 +407,11 @@ void DealerScene::openGame(QDomDocument &doc)
     }
     setGameState( dealer.attribute("data") );
 
-    if (undoList.count() > 1) {
-        setState(undoList.takeLast());
+    if (d->undoList.count() > 1) {
+        setState(d->undoList.takeLast());
         takeState(); // copying it again
-        emit undoPossible(undoList.count() > 1);
+        eraseRedo();
+        emit undoPossible(d->undoList.count() > 1);
     }
 
     emit updateMoves();
@@ -375,14 +422,14 @@ void DealerScene::undo()
 {
     unmarkAll();
     stopDemo();
-    kDebug(11111) << "::undo" << undoList.count();
-    if (undoList.count() > 1) {
-        delete undoList.last();
-        undoList.removeLast(); // the current state
-        setState(undoList.takeLast());
+    kDebug(11111) << "::undo" << d->undoList.count();
+    if (d->undoList.count() > 1) {
+        d->redoList.append( d->undoList.takeLast() );
+        setState(d->undoList.takeLast());
         emit updateMoves();
         takeState(); // copying it again
-        emit undoPossible(undoList.count() > 1);
+        emit undoPossible(d->undoList.count() > 1);
+        emit redoPossible(d->redoList.count() > 0);
         if ( d->toldAboutLostGame ) { // everything's possible again
             hintPossible( true );
             demoPossible( true );
@@ -393,6 +440,33 @@ void DealerScene::undo()
     emit gameSolverUnknown();
 }
 
+void DealerScene::redo()
+{
+    unmarkAll();
+    stopDemo();
+    kDebug(11111) << "::redo" << d->redoList.count();
+    if (d->redoList.count() > 0) {
+        setState(d->redoList.takeLast());
+        emit updateMoves();
+        takeState(); // copying it again
+        emit undoPossible(d->undoList.count() > 1);
+        emit redoPossible(d->redoList.count() > 0);
+        if ( d->toldAboutLostGame ) { // everything's possible again
+            hintPossible( true );
+            demoPossible( true );
+            d->toldAboutLostGame = false;
+            d->toldAboutWonGame = false;
+        }
+    }
+    emit gameSolverUnknown();
+}
+
+void DealerScene::eraseRedo()
+{
+    qDeleteAll(d->redoList);
+    d->redoList.clear();
+    emit redoPossible( false );
+}
 
 // ================================================================
 //                         class DealerScene
@@ -413,6 +487,7 @@ DealerScene::DealerScene():
     d->myActions = 0;
     d->m_solver = 0;
     d->m_solver_thread = 0;
+    d->offx = d->offy = 0;
     d->neededFutureMoves = 1;
     d->toldAboutLostGame = false;
     d->toldAboutWonGame = false;
@@ -425,9 +500,19 @@ DealerScene::DealerScene():
     d->demotimer = new QTimer(this);
     connect(d->demotimer, SIGNAL(timeout()), SLOT(demo()));
     d->m_autoDropFactor = 1;
-    connect( this, SIGNAL( gameWon( bool ) ), SLOT( slotShowGame(bool) ) );
+    d->m_autoDropOnce = false;
+    connect( this, SIGNAL( gameWon( bool ) ), SLOT( showWonMessage() ) );
 
     d->backPix.load( KStandardDirs::locateLocal( "data", "kpat/back.png" ) );
+
+    d->stateTimer = new QTimer( this );
+    connect( d->stateTimer, SIGNAL( timeout() ), this, SLOT( takeState() ) );
+    d->stateTimer->setInterval( 100 );
+    d->stateTimer->setSingleShot( true );
+
+    d->autoDropTimer = new QTimer( this );
+    connect( d->autoDropTimer, SIGNAL( timeout() ), this, SLOT( startAutoDrop() ) );
+    d->autoDropTimer->setSingleShot( true );
 }
 
 DealerScene::~DealerScene()
@@ -454,6 +539,10 @@ DealerScene::~DealerScene()
     d->m_solver_thread = 0;
     delete d->m_solver;
     d->m_solver = 0;
+    qDeleteAll( d->undoList );
+    qDeleteAll( d->redoList );
+
+    delete d;
 }
 
 
@@ -462,6 +551,16 @@ DealerScene::~DealerScene()
 
 void DealerScene::hint()
 {
+    kDebug() << waiting() << d->stateTimer->isActive();
+    if ( waiting() && d->stateTimer->isActive() ) {
+        QTimer::singleShot( 150, this, SLOT( hint() ) );
+        return;
+    }
+
+    d->autoDropTimer->stop();
+
+    setWaiting( true );
+
     unmarkAll();
     clearHints();
     kDebug(11111) << "hint" << d->winMoves.count();
@@ -486,9 +585,11 @@ void DealerScene::hint()
     } else
         getHints();
 
-    for (HintList::ConstIterator it = hints.begin(); it != hints.end(); ++it)
+    for (HintList::ConstIterator it = hints.constBegin(); it != hints.constEnd(); ++it)
         mark((*it)->card());
     clearHints();
+
+    setWaiting( false );
 }
 
 void DealerScene::getSolverHints()
@@ -594,14 +695,14 @@ bool DealerScene::checkPrefering( int /*checkIndex*/, const Pile *, const CardLi
 
 void DealerScene::clearHints()
 {
-    for (HintList::Iterator it = hints.begin(); it != hints.end(); ++it)
-        delete *it;
+    qDeleteAll( hints );
     hints.clear();
 }
 
 void DealerScene::newHint(MoveHint *mh)
 {
-    //kDebug() << "newHint" << mh->pile()->objectName() << " " << mh->card()->name() << " " << mh->priority();
+    //kDebug() << ( void* )mh << mh->pile()->objectName() <<
+    //mh->card()->rank() << mh->card()->suit() << mh->priority();
     hints.append(mh);
 }
 
@@ -622,57 +723,23 @@ void DealerScene::mouseMoveEvent(QGraphicsSceneMouseEvent* e)
         (*it)->setPos( ( *it )->pos() + e->scenePos() - moving_start );
     }
 
-    PileList sources;
-    QList<QGraphicsItem *> list = collidingItems( movingCards.first() );
-
-    // kDebug() << "movingCards" << movingCards.first()->sceneBoundingRect();
-    for (QList<QGraphicsItem *>::Iterator it = list.begin(); it != list.end(); ++it)
-    {
-        //kDebug() << "it" << ( *it )->type() << " " << ( *it )->sceneBoundingRect();
-        if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId) {
-            Card *c = dynamic_cast<Card*>(*it);
-            assert(c);
-            if (!c->isFaceUp())
-                continue;
-            if (c->source() == movingCards.first()->source())
-                continue;
-            if (sources.indexOf(c->source()) != -1)
-                continue;
-            sources.append(c->source());
-        } else {
-            if ((*it)->type() == QGraphicsItem::UserType + DealerScene::PileTypeId ) {
-                Pile *p = static_cast<Pile*>(*it);
-                if (p->isEmpty() && !sources.contains(p))
-                    sources.append(p);
-            } else {
-                kDebug(11111) << "unknown object" << *it << " " << (*it)->type();
-            }
-        }
-    }
-
     // TODO some caching of the results
     unmarkAll();
 
-    for (PileList::Iterator it = sources.begin(); it != sources.end(); ++it)
-    {
-        bool b = (*it)->legalAdd(movingCards);
-        // kDebug() << "legalAdd" << b << " " << ( *it )->x();
-        if (b) {
-            if ((*it)->isEmpty()) {
-                (*it)->setHighlighted(true);
-                marked.append(*it);
-            } else {
-                mark((*it)->top());
-            }
+    Pile * dropPile = targetPile();
+    if (dropPile) {
+        if (dropPile->isEmpty()) {
+            dropPile->setHighlighted(true);
+            marked.append(dropPile);
+        } else {
+            mark(dropPile->top());
         }
     }
-
     moving_start = e->scenePos();
 }
 
 void DealerScene::mark(Card *c)
 {
-//    kDebug() << "mark" << c->name();
     c->setHighlighted(true);
     if (!marked.contains(c))
         marked.append(c);
@@ -768,8 +835,10 @@ void DealerScene::startNew()
     delete d->wonItem;
     d->wonItem = 0;
     d->gothelp = false;
-    qDeleteAll(undoList);
-    undoList.clear();
+    qDeleteAll(d->undoList);
+    d->undoList.clear();
+    qDeleteAll(d->redoList);
+    d->redoList.clear();
     d->toldAboutLostGame = false;
     d->toldAboutWonGame = false;
     emit updateMoves();
@@ -796,51 +865,12 @@ void DealerScene::startNew()
     update();
 }
 
-void DealerScene::slotShowGame(bool gothelp)
+void DealerScene::showWonMessage()
 {
-    kDebug(11111) << "slotShowGame" << waiting();
+    kDebug(11111) << "showWonMessage" << waiting();
     emit undoPossible( false );
 
-    int wx = qRound( width() * 0.7 );
-    int wy = qRound( height() * 0.6 );
-
-    QImage img = QImage( wx, wy, QImage::Format_ARGB32 );
-    img.fill( qRgba( 0, 0, 255, 0 ) );
-    QPainter p( &img );
-
-    QSvgRenderer renderer( KStandardDirs::locate( "data", "kpat/won.svg" ) );
-    renderer.render( &p, "frame", QRect( 5, 5, wx-10, wy-10 ) );
-
-    QString text = i18n( "Congratulations! You have won." );
-    if ( gothelp )
-        text = i18n( "Congratulations! We have won." );
-    QFont font;
-    font.setPointSize( 36 );
-
-    int twidth = QFontMetrics( font ).width( text );
-    int fontsize = 36;
-    while ( twidth > wx * 0.9 )
-    {
-        fontsize--;
-        font.setPointSize( fontsize );
-        twidth = QFontMetrics( font ).width ( text );
-    }
-
-    p.setFont( font );
-    p.setPen( Qt::white );
-    p.drawText( int( ( wx - twidth ) / 2 ),
-                int( ( wy - QFontMetrics( font ).descent() ) / 2 ),
-                text );
-    p.end();
-
-    QGraphicsPixmapItem *item = new QGraphicsPixmapItem( 0, this );
-    item->setPixmap( QPixmap::fromImage( img ) );
-    item->setZValue( 2000 );
-
-    d->wonItem = item;
-
-    d->wonItem->setPos( QPointF( ( width() - d->wonItem->sceneBoundingRect().width() ) / 2,
-                              ( height() - d->wonItem->sceneBoundingRect().height() ) / 2 ) );
+    updateWonItem();
 
     emit demoPossible( false );
     emit hintPossible( false );
@@ -848,6 +878,68 @@ void DealerScene::slotShowGame(bool gothelp)
     // emit undoPossible(false); // technically it's possible but cheating :)
 }
 
+void DealerScene::updateWonItem()
+{
+    QSvgRenderer renderer( KStandardDirs::locate( "data", "kpat/won.svg" ) );
+
+    const QSizeF size = renderer.boundsOnElement("frame").size();
+    const qreal aspectRatio = size.width() / size.height();
+    int boxWidth;
+    int boxHeight;
+    if (width() < aspectRatio * height())
+    {
+        boxWidth = width() * 0.7;
+        boxHeight = boxWidth / aspectRatio;
+    }
+    else
+    {
+        boxHeight = height() * 0.7;
+        boxWidth = boxHeight * aspectRatio;
+    }
+
+    // Only regenerate the pixmap if it doesn't already exist or the new one is a significantly different size.
+    if ( !d->wonItem || abs( d->wonItem->boundingRect().width() - boxWidth ) > 20 )
+    {
+        if ( !d->wonItem )
+        {
+            d->wonItem  = new QGraphicsPixmapItem( 0, this );
+            d->wonItem->setZValue( 2000 );
+        }
+
+        QRect contentsRect = QRect( 0, 0, boxWidth, boxHeight );
+        QPixmap pix( contentsRect.size() );
+        pix.fill( Qt::transparent );
+        QPainter p( &pix );
+        renderer.render( &p, "frame", contentsRect );
+
+        QString text = i18n( "Congratulations! You have won." );
+        if ( d->gothelp )
+            text = i18n( "Congratulations! We have won." );
+
+        QFont font;
+
+        int fontsize = 36;
+        font.setPointSize( fontsize );
+        int twidth = QFontMetrics( font ).width( text );
+        while ( twidth > boxWidth * 0.9 && fontsize > 5 )
+        {
+            fontsize--;
+            font.setPointSize( fontsize );
+            twidth = QFontMetrics( font ).width ( text );
+        }
+
+        p.setFont( font );
+        p.setPen( Qt::white );
+        p.drawText( contentsRect, Qt::AlignCenter, text );
+        p.end();
+
+        d->wonItem->setPixmap( pix );
+    }
+
+    d->wonItem->setPos( QPointF( ( width() - d->wonItem->sceneBoundingRect().width() ) / 2,
+                                 ( height() - d->wonItem->sceneBoundingRect().height() ) / 2 ) );
+}
+
 void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent *e )
 {
     QGraphicsScene::mouseReleaseEvent( e );
@@ -869,6 +961,7 @@ void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent *e )
                     countGame();
                 }
                 takeState();
+                eraseRedo();
             }
             return;
         }
@@ -877,17 +970,34 @@ void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent *e )
             assert(c);
             pileClicked(c);
             takeState();
+            eraseRedo();
             return;
         }
     }
 
-    if (!movingCards.count())
+    if (movingCards.isEmpty())
         return;
-    Card *c = static_cast<Card*>(movingCards.first());
-    assert(c);
 
     unmarkAll();
 
+    Pile * destination = targetPile();
+    if (destination) {
+        Card *c = static_cast<Card*>(movingCards.first());
+        assert(c);
+        countGame();
+        c->source()->moveCards(movingCards, destination);
+        takeState();
+        eraseRedo();
+    }
+    else
+    {
+        movingCards.first()->source()->moveCardsBack(movingCards);
+    }
+    movingCards.clear();
+}
+
+Pile * DealerScene::targetPile()
+{
     QList<QGraphicsItem *> list = collidingItems( movingCards.first() );
     HitList sources;
 
@@ -947,9 +1057,7 @@ void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent *e )
             ++it;
     }
 
-    if (sources.isEmpty()) {
-        c->source()->moveCardsBack(movingCards);
-    } else {
+    if (!sources.isEmpty()) {
         HitList::Iterator best = sources.begin();
         HitList::Iterator it = best;
         for (++it; it != sources.end(); ++it )
@@ -961,11 +1069,10 @@ void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent *e )
                 best = it;
             }
         }
-        countGame();
-        c->source()->moveCards(movingCards, (*best).source);
-        takeState();
+        return (*best).source;
+    } else {
+        return 0;
     }
-    movingCards.clear();
 }
 
 void DealerScene::mouseDoubleClickEvent( QGraphicsSceneMouseEvent *e )
@@ -994,6 +1101,7 @@ void DealerScene::mouseDoubleClickEvent( QGraphicsSceneMouseEvent *e )
         countGame();
     }
     takeState();
+    eraseRedo();
 }
 
 bool DealerScene::cardClicked(Card *c) {
@@ -1033,7 +1141,7 @@ State *DealerScene::getState()
     QList<QGraphicsItem *> list = items();
     State * st = new State;
 
-    for (QList<QGraphicsItem *>::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for (QList<QGraphicsItem *>::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
     {
        if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId ) {
            Card *c = dynamic_cast<Card*>(*it);
@@ -1079,7 +1187,7 @@ void DealerScene::setState(State *st)
         }
     }
 
-    for (CardStateList::ConstIterator it = n->begin(); it != n->end(); ++it)
+    for (CardStateList::ConstIterator it = n->constBegin(); it != n->constEnd(); ++it)
     {
         Card *c = (*it).it;
         //c->stopAnimation();
@@ -1093,12 +1201,12 @@ void DealerScene::setState(State *st)
         c->setTakenDown(s.tookdown || (target && !s.source->target()));
     }
 
-    for (PileList::ConstIterator it = piles.begin(); it != piles.end(); ++it)
+    for (PileList::ConstIterator it = piles.constBegin(); it != piles.constEnd(); ++it)
     {
         ( *it )->relayoutCards();
     }
 
-    for (CardStateList::ConstIterator it = n->begin(); it != n->end(); ++it)
+    for (CardStateList::ConstIterator it = n->constBegin(); it != n->constEnd(); ++it)
     {
         Card *c = (*it).it;
         c->stopAnimation();
@@ -1117,7 +1225,7 @@ Pile *DealerScene::findTarget(Card *c)
 
     CardList empty;
     empty.append(c);
-    for (PileList::ConstIterator it = piles.begin(); it != piles.end(); ++it)
+    for (PileList::ConstIterator it = piles.constBegin(); it != piles.constEnd(); ++it)
     {
         if (!(*it)->target())
             continue;
@@ -1129,7 +1237,7 @@ Pile *DealerScene::findTarget(Card *c)
 
 void DealerScene::setWaiting(bool w)
 {
-    kDebug(11111) << "setWaiting" << w << " " << _waiting;
+    //kDebug(11111) << "setWaiting" << w << " " << _waiting;
     assert( _waiting > 0 || w );
 
     if (w)
@@ -1142,28 +1250,41 @@ void DealerScene::setWaiting(bool w)
 void DealerScene::setAutoDropEnabled(bool a)
 {
     _autodrop = a;
-    QTimer::singleShot(TIME_BETWEEN_MOVES, this, SLOT(startAutoDrop()));
+    d->autoDropTimer->start( TIME_BETWEEN_MOVES );
+}
+
+void DealerScene::slotAutoDrop()
+{
+    d->m_autoDropOnce = true;
+    d->autoDropTimer->start( TIME_BETWEEN_MOVES );
+}
+
+void DealerScene::setSolverEnabled(bool a)
+{
+    _usesolver = a;
+    if(_usesolver)
+        startSolver();
 }
 
 bool DealerScene::startAutoDrop()
 {
-    if (!autoDrop())
+    if (!autoDrop() && !d->m_autoDropOnce)
         return false;
 
-    if (!movingCards.isEmpty()) {
-        QTimer::singleShot( speedUpTime( TIME_BETWEEN_MOVES ), this, SLOT(startAutoDrop()));
+    if (!movingCards.isEmpty() || waiting() ) {
+        d->autoDropTimer->start( speedUpTime( TIME_BETWEEN_MOVES ) );
         return true;
     }
 
     QList<QGraphicsItem *> list = items();
 
     //kDebug( 11111 ) << "startAutoDrop\n";
-    for (QList<QGraphicsItem *>::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for (QList<QGraphicsItem *>::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
     {
         if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId ) {
             Card *c = static_cast<Card*>(*it);
             if (c->animated()) {
-                QTimer::singleShot( speedUpTime( TIME_BETWEEN_MOVES ), this, SLOT(startAutoDrop()));
+                d->autoDropTimer->start( speedUpTime( TIME_BETWEEN_MOVES ) );
                 //kDebug(11111) << "animation still going on\n";
                 return true;
             }
@@ -1172,10 +1293,9 @@ bool DealerScene::startAutoDrop()
 
     kDebug(11111) << gettime() << "startAutoDrop \n";
 
-    unmarkAll();
     clearHints();
     getHints();
-    for (HintList::ConstIterator it = hints.begin(); it != hints.end(); ++it) {
+    for (HintList::ConstIterator it = hints.constBegin(); it != hints.constEnd(); ++it) {
         MoveHint *mh = *it;
         if (mh->pile() && mh->pile()->target() && mh->priority() > 120 && !mh->card()->takenDown()) {
             //setWaiting(true);
@@ -1197,6 +1317,9 @@ bool DealerScene::startAutoDrop()
             t->source()->moveCards(cards, mh->pile());
             int count = 0;
 
+            if ( !cards.isEmpty() )
+                unmarkAll();
+
             for ( CardList::Iterator it2 = cards.begin(); it2 != cards.end(); ++it2 )
             {
                 Card *t = *it2;
@@ -1221,6 +1344,7 @@ bool DealerScene::startAutoDrop()
     }
     clearHints();
     d->m_autoDropFactor = 1;
+    d->m_autoDropOnce = false;
 
     return false;
 }
@@ -1243,12 +1367,12 @@ void DealerScene::stopAndRestartSolver()
     }
 
     QList<QGraphicsItem *> list = items();
-    for (QList<QGraphicsItem *>::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for (QList<QGraphicsItem *>::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
     {
         if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId ) {
             Card *c = static_cast<Card*>(*it);
             if (c->animated()) {
-                d->updateSolver->start();
+                startSolver();
                 //kDebug(11111) << "animation still going on\n";
                 return;
             }
@@ -1271,7 +1395,7 @@ void DealerScene::slotSolverEnded()
         d->m_solver_thread = new SolverThread( d->m_solver );
         connect( d->m_solver_thread, SIGNAL( finished() ), this, SLOT( slotSolverFinished() ));
     }
-    d->m_solver_thread->start(QThread::IdlePriority);
+    d->m_solver_thread->start(_usesolver ? QThread::IdlePriority : QThread::NormalPriority );
 }
 
 void DealerScene::slotSolverFinished()
@@ -1281,7 +1405,7 @@ void DealerScene::slotSolverFinished()
     {
         // if we can lock, then this finish signal is abnormal
         d->solverMutex.unlock();
-        d->updateSolver->start();
+        startSolver();
         return;
     }
 
@@ -1303,7 +1427,7 @@ void DealerScene::slotSolverFinished()
         break;
     case Solver::QUIT:
         d->solverMutex.unlock();
-        d->updateSolver->start();
+        startSolver();
         break;
     case Solver::MLIMIT:
         d->solverMutex.unlock();
@@ -1317,6 +1441,7 @@ void DealerScene::waitForAutoDrop(Card * c) {
     c->disconnect();
     c->stopAnimation(); // should be a no-op
     takeState();
+    eraseRedo();
 }
 
 void DealerScene::waitForWonAnim(Card *c)
@@ -1346,7 +1471,7 @@ void DealerScene::rescale(bool onlypiles)
 {
     if ( !onlypiles )
         Deck::deck()->update();
-    for (PileList::ConstIterator it = piles.begin(); it != piles.end(); ++it)
+    for (PileList::ConstIterator it = piles.constBegin(); it != piles.constEnd(); ++it)
     {
         ( *it )->rescale();
     }
@@ -1356,8 +1481,10 @@ void DealerScene::setGameNumber(long gmn)
 {
     // Deal in the range of 1 to INT_MAX.
     gamenumber = ((gmn < 1) ? 1 : ((gmn > 0x7FFFFFFF) ? 0x7FFFFFFF : gmn));
-    qDeleteAll(undoList);
-    undoList.clear();
+    qDeleteAll(d->undoList);
+    d->undoList.clear();
+    qDeleteAll(d->redoList);
+    d->redoList.clear();
 }
 
 void DealerScene::addPile(Pile *p)
@@ -1385,7 +1512,7 @@ void DealerScene::stopDemo()
     } else d->stop_demo_next = false;
 
     QList<QGraphicsItem *> list = items();
-    for (QList<QGraphicsItem *>::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for (QList<QGraphicsItem *>::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
     {
         if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId ) {
             Card *c = static_cast<Card*>(*it);
@@ -1444,11 +1571,12 @@ void DealerScene::won()
     if (n>m)
         kc.writeEntry(QString("maxwinstreak%1").arg(d->_id),n);
     kc.writeEntry(QString("loosestreak%1").arg(d->_id),0);
+    kc.sync();
 
     // sort cards by increasing z
     QList<QGraphicsItem *> list = items();
     QList<CardPtr> cards;
-    for (QList<QGraphicsItem *>::ConstIterator it=list.begin(); it!=list.end(); ++it)
+    for (QList<QGraphicsItem *>::ConstIterator it=list.constBegin(); it!=list.constEnd(); ++it)
         if ((*it)->type() == QGraphicsItem::UserType + DealerScene::CardTypeId ) {
             CardPtr p;
             p.ptr = dynamic_cast<Card*>(*it);
@@ -1506,8 +1634,8 @@ MoveHint *DealerScene::chooseHint()
 
 void DealerScene::demo()
 {
-    kDebug(11111) << "demo" << waiting();
-    if ( waiting() )
+    kDebug(11111) << "demo" << waiting() << d->stateTimer->isActive();
+    if ( waiting() || d->stateTimer->isActive() )
         return;
 
     if (d->stop_demo_next) {
@@ -1526,7 +1654,7 @@ void DealerScene::demo()
     MoveHint *mh = chooseHint();
     if (mh) {
         kDebug(11111) << "moveFrom" << mh->card()->source()->objectName();
-        assert(mh->card()->source() == Deck::deck() ||
+        myassert(mh->card()->source() == Deck::deck() ||
                mh->card()->source()->legalRemove(mh->card(), true));
 
         CardList empty;
@@ -1539,7 +1667,7 @@ void DealerScene::demo()
                 empty.append(*it);
         }
 
-        assert(!empty.isEmpty());
+        myassert(!empty.isEmpty());
 
         qreal *oldcoords = new qreal[2*empty.count()];
         int i = 0;
@@ -1595,6 +1723,7 @@ void DealerScene::demo()
 
     emit demoActive( true );
     takeState();
+    eraseRedo();
 }
 
 Card *DealerScene::demoNewCards()
@@ -1605,10 +1734,11 @@ Card *DealerScene::demoNewCards()
 void DealerScene::newDemoMove(Card *m)
 {
     kDebug(11111) << "newDemoMove" << m->rank() << " " << m->suit();
-    setWaiting( true );
     if ( m->animated() )
+    {
         connect(m, SIGNAL(stoped(Card*)), SLOT(waitForDemo(Card*)));
-    else
+        setWaiting( true );
+    } else
         waitForDemo( 0 );
 }
 
@@ -1618,13 +1748,10 @@ void DealerScene::waitForDemo(Card *t)
     {
         kDebug(11111) << "waitForDemo" << t->rank() << " " << t->suit();
         t->disconnect(this, SLOT( waitForDemo( Card* ) ) );
+        setWaiting( false );
+        takeState();
     }
-    setWaiting( false );
-    if ( !waiting() )
-    {
-        d->demotimer->setSingleShot( true );
-        d->demotimer->start(250);
-    }
+    d->demotimer->start(250);
 }
 
 void DealerScene::setSolver( Solver *s) {
@@ -1636,7 +1763,7 @@ void DealerScene::setSolver( Solver *s) {
 
 bool DealerScene::isGameWon() const
 {
-    for (PileList::ConstIterator it = piles.begin(); it != piles.end(); ++it)
+    for (PileList::ConstIterator it = piles.constBegin(); it != piles.constEnd(); ++it)
     {
         if (!(*it)->target() && !(*it)->isEmpty())
             return false;
@@ -1644,6 +1771,12 @@ bool DealerScene::isGameWon() const
     return true;
 }
 
+void DealerScene::startSolver() const
+{
+    if(_usesolver)
+        d->updateSolver->start();
+}
+
 void DealerScene::unlockSolver() const
 {
     d->solverMutex.unlock();
@@ -1691,6 +1824,7 @@ void DealerScene::countGame()
         ++Total;
         kc.writeEntry(QString("total%1").arg(d->_id),Total);
         d->_gameRecorded = true;
+	kc.sync();
     }
 }
 
@@ -1705,13 +1839,14 @@ void DealerScene::countLoss()
         if (n>m)
             kc.writeEntry(QString("maxloosestreak%1").arg(d->_id),n);
         kc.writeEntry(QString("winstreak%1").arg(d->_id),0);
+        kc.sync();
     }
 }
 
 QPointF DealerScene::maxPilePos() const
 {
     QPointF maxp( 0, 0 );
-    for (PileList::ConstIterator it = piles.begin(); it != piles.end(); ++it)
+    for (PileList::ConstIterator it = piles.constBegin(); it != piles.constEnd(); ++it)
     {
         // kDebug() << ( *it )->objectName() << " " <<  ( *it )->pilePos() << " " << ( *it )->reservedSpace();
         // we call fabs here, because reserved space of -50 means for a pile at -1x-1 means it wants at least
@@ -1745,15 +1880,22 @@ void DealerScene::setSceneSize( const QSize &s )
 
     qreal scaleX = s.width() / cardMap::self()->wantedCardWidth() / ( defaultSceneRect.x() + 2 );
     qreal scaleY = s.height() / cardMap::self()->wantedCardHeight() / ( defaultSceneRect.y() + 2 );
-    //kDebug() << "scaleY" << scaleY << " " << scaleX;
+    //kDebug() << "scales" << scaleY << scaleX;
     qreal n_scaleFactor = qMin(scaleX, scaleY);
 
+    d->offx = 0;
+    d->offy = 0; // just for completness
+    if ( n_scaleFactor < scaleX )
+    {
+        d->offx = ( s.width() - ( ( defaultSceneRect.x() + 2 ) * n_scaleFactor ) * cardMap::self()->wantedCardWidth() ) / 2;
+    }
+
     cardMap::self()->setWantedCardWidth( int( n_scaleFactor * 10 * cardMap::self()->wantedCardWidth() ) );
 
     for (PileList::Iterator it = piles.begin(); it != piles.end(); ++it)
     {
         Pile *p = *it;
-        if ( p->pilePos().x() < 0 || p->pilePos().y() < 0 )
+        if ( p->pilePos().x() < 0 || p->pilePos().y() < 0 || d->offx > 0 )
             p->rescale();
         QSizeF ms( cardMap::self()->wantedCardWidth(),
                    cardMap::self()->wantedCardHeight() );
@@ -1790,7 +1932,7 @@ void DealerScene::setSceneSize( const QSize &s )
 
         //kDebug() << p->objectName() << " " << p->spread() << " " << myRect;
 
-        for ( PileList::ConstIterator it2 = piles.begin(); it2 != piles.end(); ++it2 )
+        for ( PileList::ConstIterator it2 = piles.constBegin(); it2 != piles.constEnd(); ++it2 )
         {
             if ( *it2 == p || !( *it2 )->isVisible() )
                 continue;
@@ -1802,11 +1944,11 @@ void DealerScene::setSceneSize( const QSize &s )
                 pileRect.moveBottom( (*it2)->y() );
 
 
-            //kDebug() << "compa" << p->objectName() << " " << myRect << " " << ( *it2 )->objectName() << " " << pileRect << " " << myRect.intersects( pileRect );
+            //kDebug() << "compa" << p->objectName() << myRect << ( *it2 )->objectName() << pileRect << myRect.intersects( pileRect );
             // if the growing pile intersects with another pile, we need to solve the conflict
             if ( myRect.intersects( pileRect ) )
             {
-                //kDebug() << "compa" << p->objectName() << " " << myRect << " " << ( *it2 )->objectName() << " " << pileRect << " " << myRect.intersects( pileRect );
+                //kDebug() << "compa" << p->objectName() << myRect << ( *it2 )->objectName() << pileRect << myRect.intersects( pileRect );
                 QSizeF pms = ( *it2 )->maximalSpace();
 
                 if ( p->reservedSpace().width() != 10 )
@@ -1844,19 +1986,19 @@ void DealerScene::setSceneSize( const QSize &s )
                             Q_ASSERT( false ); // TODO
                             myRect.setLeft( ( *it2 )->x() + cardMap::self()->wantedCardWidth() + 1);
                             //kDebug() << "5. reduced height of" << p->objectName();
-                        } else {
+                        } else if ( ( *it2 )->y() >= 1  ) {
                             myRect.setBottom( ( *it2 )->y() - 1 );
-                            //kDebug() << "6. reduced height of" << p->objectName();
+                            //kDebug() << "6. reduced height of" << p->objectName() << (*it2)->y() - 1 << myRect;
                         }
                 }
 
-                //kDebug() << pms << " " << cardMap::self()->wantedCardWidth() << " " << cardMap::self()->wantedCardHeight();
+                kDebug() << pms  << cardMap::self()->wantedCardWidth()  << cardMap::self()->wantedCardHeight();
                 Q_ASSERT( pms.width() >= cardMap::self()->wantedCardWidth() - 0.1 );
                 Q_ASSERT( pms.height() >= cardMap::self()->wantedCardHeight() - 0.1 );
                 ( *it2 )->setMaximalSpace( pms );
             }
         }
-        //kDebug() << myRect << " " << cardMap::self()->wantedCardHeight();
+        kDebug() << myRect << cardMap::self()->wantedCardHeight();
         Q_ASSERT( myRect.width() >= cardMap::self()->wantedCardWidth() - 0.1 );
         Q_ASSERT( myRect.height() >= cardMap::self()->wantedCardHeight() - 0.1 );
 
@@ -1864,8 +2006,7 @@ void DealerScene::setSceneSize( const QSize &s )
     }
 
     if ( d->wonItem )
-        d->wonItem->setPos( QPointF( ( width() - d->wonItem->sceneBoundingRect().width() ) / 2,
-                                     ( height() - d->wonItem->sceneBoundingRect().height() ) / 2 ) );
+        updateWonItem();
 
     for (PileList::Iterator it = piles.begin(); it != piles.end(); ++it)
     {
@@ -1884,18 +2025,21 @@ Solver *DealerScene::solver() const { return d->m_solver; }
 int DealerScene::neededFutureMoves() const { return d->neededFutureMoves; }
 void DealerScene::setNeededFutureMoves( int i ) { d->neededFutureMoves = i; }
 
+qreal DealerScene::offsetX() const { return d->offx; }
+qreal DealerScene::offsetY() const { return d->offy; }
+
 QSvgRenderer *DealerScene::DealerScenePrivate::backren = 0;
 
 void DealerScene::drawBackground ( QPainter * painter, const QRectF & rect )
 {
-    kDebug(11111) << "drawBackground" << rect << " " << d->hasScreenRect;
+    kDebug(11111) << "drawBackground" << rect << d->hasScreenRect;
     if ( !d->hasScreenRect )
         return;
 
     if ( ( d->backPix.width() != width() ) || ( d->backPix.height() != height() ) )
     {
         if ( !d->backren )
-            d->backren = new QSvgRenderer( KStandardDirs::locate( "data", "kpat/backgrounds/background.svg" ) );
+            d->backren = new QSvgRenderer( KStandardDirs::locate( "data", "kpat/background.svg" ) );
 
         QImage img( width(), height(), QImage::Format_ARGB32);
         QPainter p2( &img );
@@ -1908,4 +2052,64 @@ void DealerScene::drawBackground ( QPainter * painter, const QRectF & rect )
     painter->drawPixmap( 0, 0, d->backPix );
 }
 
+QString DealerScene::save_it()
+{
+    KTemporaryFile file;
+    file.setAutoRemove(false);
+    file.open();
+    QDomDocument doc("kpat");
+    saveGame(doc);
+    QTextStream stream (&file);
+    stream << doc.toString();
+    stream.flush();
+    file.flush();
+    file.close();
+    return file.fileName();
+}
+
+void DealerScene::createDump( QPaintDevice *device )
+{
+    int maxz = 0;
+    QList<QGraphicsItem *> list = items();
+    for (int i = 0; i < list.size(); ++i) {
+        QGraphicsItem *item = list.at(i);
+        assert( item->zValue() >= 0 );
+        maxz = qMax( maxz, int( item->zValue() ) );
+    }
+    QStyleOptionGraphicsItem options;
+    QPainter p( device );
+
+    for ( int z = 0; z <= maxz; ++z )
+        for (int i = 0; i < list.size(); ++i)
+        {
+            QGraphicsItem *item = list.at(i);
+            if ( !item->isVisible() || item->zValue() != z )
+                continue;
+
+            if (item->type() == QGraphicsItem::UserType + DealerScene::CardTypeId )
+            {
+
+            }
+            else if (item->type() == QGraphicsItem::UserType + DealerScene::PileTypeId )
+            {
+                QGraphicsPixmapItem *pitem = dynamic_cast<QGraphicsPixmapItem*>( item );
+                assert( pitem );
+                p.save();
+                p.setTransform(item->deviceTransform(p.worldTransform()), false);
+                //item->paint( &p, &options );
+                p.drawPixmap( 0, 0, pitem->pixmap() );
+                p.restore();
+                continue;
+            } else
+                assert( false );
+
+            p.save();
+            p.setTransform(item->deviceTransform(p.worldTransform()), false);
+            item->paint( &p, &options );
+            p.restore();
+        }
+}
+
+void DealerScene::mapOldId(int) {}
+
 #include "dealer.moc"
