@@ -19,11 +19,22 @@
 ---------------------------------------------------------------------------*/
 
 #include "spider.h"
-#include <klocale.h>
+#include "version.h"
+#include "view.h"
+#include "speeds.h"
 #include "deck.h"
-#include <kdebug.h>
 #include "patsolve/spider.h"
 
+#include <QApplication>
+
+#include <kdebug.h>
+#include <klocale.h>
+#include <kselectaction.h>
+#include <kxmlguiwindow.h>
+#include <kactioncollection.h>
+#include <kxmlguifactory.h>
+#include <kconfiggroup.h>
+
 void SpiderPile::moveCards(CardList &c, Pile *to)
 {
     Pile::moveCards(c, to);
@@ -48,11 +59,14 @@ void SpiderPile::moveCards(CardList &c, Pile *to)
 
 //-------------------------------------------------------------------------//
 
-Spider::Spider(int suits)
+Spider::Spider()
     : DealerScene(), m_leg(0), m_redeal(0)
 {
     const qreal dist_x = 11.2;
 
+    KConfigGroup cg(KGlobal::config(), settings_group );
+    int suits = cg.readEntry( "SpiderSuits", 2);
+
     Deck::create_deck(this, 2, suits);
 
     // I deal the cards into 'redeal' piles, so hide the deck
@@ -106,6 +120,79 @@ Spider::Spider(int suits)
     setAutoDropEnabled(false);
     setActions(DealerScene::Hint | DealerScene::Demo );
     setSolver( new SpiderSolver( this ) );
+
+    options = new KSelectAction(i18n("Options"), this );
+    KXmlGuiWindow *xmlgui = 0;
+
+    if ( PatienceView::instance() )
+    {
+        xmlgui = PatienceView::instance()->parent();
+
+        xmlgui->actionCollection()->addAction("dealer_options", options);
+    }
+
+    options->addAction( "1 Suit (Easy)" );
+    options->addAction( "2 Suits (Medium)" );
+    options->addAction( "4 Suits (Hard)" );
+
+    if ( suits == 1 )
+        options->setCurrentItem( 0 );
+    else if ( suits == 2 )
+        options->setCurrentItem( 1 );
+    else
+        options->setCurrentItem( 2 );
+
+    QList<QAction*> actionlist;
+    actionlist.append( options );
+
+    if ( xmlgui )
+        xmlgui->guiFactory()->plugActionList( xmlgui, QString::fromLatin1("dealer_options"), actionlist);
+    connect( options, SIGNAL( triggered( int ) ), SLOT( gameTypeChanged() ) );
+}
+
+void Spider::gameTypeChanged()
+{
+    if ( waiting() )
+    {
+        QTimer::singleShot( 100, this, SLOT( gameTypeChanged() ) );
+        return;
+    }
+
+    if ( demoActive() || isGameWon()  )
+        return;
+
+    int suits = 4;
+    if ( options->currentItem() == 0 )
+        suits = 1;
+    if ( options->currentItem() == 1 )
+        suits = 2;
+
+    setSuits( suits );
+}
+
+void Spider::setSuits(int suits)
+{
+    stopDemo();
+    unmarkAll();
+    Deck::destroy_deck();
+    Deck::create_deck(this, 2, suits);
+    KConfigGroup cg(KGlobal::config(), settings_group );
+    cg.writeEntry( "SpiderSuits", suits);
+    cg.sync();
+
+    switch ( suits )
+    {
+    case 1:
+        options->setCurrentItem( 0 );
+        break;
+    case 2:
+        options->setCurrentItem( 1 );
+        break;
+    case 3:
+        options->setCurrentItem( 2 );
+        break;
+    }
+    startNew();
 }
 
 void Spider::cardStoped(Card * t)
@@ -113,7 +200,7 @@ void Spider::cardStoped(Card * t)
     for( int column = 0; column < 10; column++ ) {
         Card *t = stack[column]->top();
         if (t && !t->isFaceUp())
-           t->flipTo(t->x(), t->y());
+           t->flipTo(t->x(), t->y(), DURATION_FLIP );
     }
     t->disconnect(this, SLOT( cardStoped( Card* ) ) );
     setWaiting( false );
@@ -321,27 +408,28 @@ void Spider::deckClicked(Card*)
     takeState();
 }
 
-static class LocalDealerInfo15 : public DealerInfo
-{
-public:
-    LocalDealerInfo15() : DealerInfo(I18N_NOOP("S&pider (Easy)"), 14) {}
-    virtual DealerScene *createGame() { return new Spider(1); }
-} ldi15;
-
-static class LocalDealerInfo16 : public DealerInfo
-{
-public:
-    LocalDealerInfo16() : DealerInfo(I18N_NOOP("Spider (&Medium)"), 15) {}
-    virtual DealerScene *createGame() { return new Spider(2); }
-} ldi16;
-
 static class LocalDealerInfo17 : public DealerInfo
 {
 public:
-    LocalDealerInfo17() : DealerInfo(I18N_NOOP("Spider (&Hard)"), 16) {}
-    virtual DealerScene *createGame() { return new Spider(4); }
+    LocalDealerInfo17() : DealerInfo(I18N_NOOP("Spider"), 17) { addOldId(14); addOldId(15); addOldId(16); }
+    virtual DealerScene *createGame() { return new Spider(); }
 } ldi17;
 
+void Spider::mapOldId(int id)
+{
+   switch (id) {
+   case 14:
+       setSuits(1);
+       break;
+   case 15:
+       setSuits(2);
+       break;
+   case 16:
+       setSuits(4);
+       break;
+   }
+}
+
 //-------------------------------------------------------------------------//
 
 #include "spider.moc"
