@@ -18,11 +18,13 @@
 
 ---------------------------------------------------------------------------*/
 
-#ifndef _SPIDER_H_
-#define _SPIDER_H_
+#ifndef SPIDER_H
+#define SPIDER_H
 
 #include "dealer.h"
 
+class KSelectAction;
+
 class SpiderPile : public Pile
 {
 public:
@@ -38,15 +40,17 @@ class Spider : public DealerScene
     Q_OBJECT
 
 public:
-    Spider(int suits);
+    Spider();
     void deal();
     void dealRow();
     bool checkPileDeck(Pile *to, bool checkForDemo = true);
     virtual void restart();
+    virtual void mapOldId(int id);
 
 public slots:
     void deckClicked(Card *c);
     void cardStoped(Card *c);
+    void gameTypeChanged();
 
 protected:
     virtual bool checkRemove(int /*checkIndex*/, const Pile *p, const Card *c) const;
@@ -54,6 +58,7 @@ protected:
     virtual QString getGameState() const;
     virtual void setGameState(const QString &stream);
     virtual Card *demoNewCards();
+    void setSuits(int s);
 
 private:
     CardList getRun(Card *c) const;
@@ -63,6 +68,8 @@ private:
     int m_leg;
     Pile *redeals[5];
     int m_redeal;
+
+    KSelectAction *options;
 };
 
 #endif
