@@ -25,16 +25,8 @@
 #include "cardmaps.h"
 #include "version.h"
 
-#include <kaction.h>
-#include <ktoggleaction.h>
-#include <kstandardgameaction.h>
-#include <kactioncollection.h>
-#include <kdebug.h>
-#include <kicon.h>
-#include <klocalizedstring.h>
-#include <krandom.h>
-#include <kxmlguifactory.h>
-#include <kxmlguiwindow.h>
+#include <cassert>
+#include <cmath>
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QList>
@@ -45,8 +37,16 @@
 #ifndef QT_NO_OPENGL
 #endif
 
-#include <assert.h>
-#include <math.h>
+#include <kaction.h>
+#include <ktoggleaction.h>
+#include <kstandardgameaction.h>
+#include <kactioncollection.h>
+#include <kdebug.h>
+#include <kicon.h>
+#include <klocalizedstring.h>
+#include <krandom.h>
+#include <kxmlguifactory.h>
+#include <kxmlguiwindow.h>
 
 DealerInfoList *DealerInfoList::_self = 0;
 
@@ -80,7 +80,8 @@ PatienceView::PatienceView( KXmlGuiWindow* _parent )
     ademo(0),
     ahint(0),
     aredeal(0),
-    m_shown( false )
+    adrop(0),
+    m_shown( true )
 {
     setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -102,6 +103,8 @@ PatienceView::~PatienceView()
 
     if (s_instance == this)
         s_instance = 0;
+
+    delete dscene();
 }
 
 KXmlGuiWindow *PatienceView::parent() const
@@ -112,7 +115,6 @@ KXmlGuiWindow *PatienceView::parent() const
 
 void PatienceView::setScene( QGraphicsScene *_scene )
 {
-    parent()->guiFactory()->unplugActionList( parent(), QString::fromLatin1("game_actions"));
     DealerScene *oldscene = dscene();
     QGraphicsView::setScene( _scene );
     resetCachedContent();
@@ -149,6 +151,8 @@ void PatienceView::setupActions()
 {
     QList<QAction*> actionlist;
 
+    parent()->guiFactory()->unplugActionList( parent(), QString::fromLatin1("game_actions"));
+
     kDebug(11111) << "setupActions" << actions();
 
     if (dscene()->actions() & DealerScene::Hint) {
@@ -172,6 +176,16 @@ void PatienceView::setupActions()
     } else
         aredeal = 0;
 
+    delete adrop;
+    if ( !dscene()->autoDrop() ) {
+        adrop = parent()->actionCollection()->addAction( "autodrop" );
+        adrop->setText( i18n("Drop!") );
+        adrop->setIcon( KIcon( "legalmoves") );
+        connect( adrop, SIGNAL( triggered( bool ) ), dscene(), SLOT( slotAutoDrop() ) );
+        actionlist.append(adrop);
+    } else
+        adrop = 0;
+
     parent()->guiFactory()->plugActionList( parent(), QString::fromLatin1("game_actions"), actionlist);
 }
 
@@ -191,9 +205,15 @@ void PatienceView::hint()
     dscene()->hint();
 }
 
+void PatienceView::setSolverEnabled(bool a)
+{
+    dscene()->setSolverEnabled( a );
+}
+
 void PatienceView::setAutoDropEnabled(bool a)
 {
     dscene()->setAutoDropEnabled( a );
+    setupActions();
 }
 
 void PatienceView::startNew()
